==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ron/db/pdodatabase.php <==
<?php
/**
 * stanislav.elbl, 23.09.2021 12:10
 */

namespace Ron\DB;

use PDO;

class PdoDatabase extends Database
{
    /**
     * @var PDO
     */
    private $pdo;

    private $connectionString;
    private $attributes;

    /** @var SqlDialect */
    private $sqlDialect;
    
    public $lobParameterDriverOptions;

    /**
     * @param string $connectionString
     * @param SqlDialect $sqlDialect
     */
    public function __construct($connectionString, $sqlDialect)
    {
        $this ->connectionString = $connectionString;
        $this ->sqlDialect = $sqlDialect;
    }


    /**
     * @return mixed
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return mixed
     */
    public function getAttributes()
    {
        return $this->attributes;
    }

    /**
     * @param mixed $attributes
     */
    public function setAttributes($attributes)
    {
        $this->attributes = $attributes;
    }



    /**
     * @return string
     */
    public function getDbType()
    {
        return $this ->sqlDialect ->getDbType();
    }


    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    public function connect($user, $password)
    {
        try {
            $this->pdo = new PDO($this->connectionString, $user, $password);
        } catch (\PDOException $e) {
            throw new DBException($this, '', $e ->getMessage(), $e, $e ->errorInfo);
        }

        if ($this ->attributes) {
            foreach ($this ->attributes as $attribute => $value)
                $this ->pdo->setAttribute($attribute, $value);
        }

        //$count = $this ->pdo ->query("SET NAMES '{$this ->codePage}'");
//        $count = $this ->pdo ->query("SET NAMES utf8");
//        if (!isset($count)) {
//            throw new \Exception('chybaaa!');
//        };
    }

    /**
     * @return bool
     */
    public function close()
    {
        return true;
    }


    public function createParameters()
    {
        return new PdoParameters($this);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return Cursor
     * @throws DBException
     */
    public function executeSelect($sqlText, $params = null)
    {
        if ($params) {
            $data = $this ->pdo ->prepare($sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }

            if (!$data ->execute($params ->getSqlParameters())) {
                throw new DBException($this, $sqlText);
            }
        } else {
            $data = $this->pdo->query($sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }
        }

        return new PdoCursor($this, $data);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @throws DBException
     */
    public function executeSql($sqlText, $params = null)
    {
        if ($params) {
            $data = $this ->pdo ->prepare($sqlText);

            if (!$data) {
                throw new DBException($this, $sqlText);
            }

            foreach ($params ->getSqlParameters() as $index => &$parameter) {
//                echo "<br/>parametr $index: "; var_dump($parameter); echo "<br/>";
                if (!isset($parameter)) {
                    $data->bindValue($index + 1, null, PDO::PARAM_NULL);
                } else if ($parameter instanceof PdoBlobParameter) {
                    $data->bindParam($index + 1, $parameter->data, PDO::PARAM_LOB, 0, $parameter ->driverOptions); //
                } else if (is_int($parameter)) {
                    $data->bindValue($index + 1, $parameter, PDO::PARAM_INT);
                } else {
                    $data->bindValue($index + 1, $parameter);
                }
            }

            if (!$data ->execute()) {
                throw new DBException($this, $sqlText);
            }

//            if (!$data ->execute($params ->getSqlParameters())) {
//                throw new DBException($this, $sqlText);
//            }

        } else {
            $count = $this->pdo->exec($sqlText);

            // count obsahuje pocet zmenenych zaznamu nebo NULL pri chybe
            if ($count === NULL ||  $count === false) {
                throw new DBException($this, $sqlText);
            }
        }
    }


    function lastAutoincrementValue()
    {
        return $this ->pdo ->lastInsertId();
    }

    /**
     * @return string
     */
    public function lastErrorText()
    {
        $result = '';

        if ($this ->pdo) {
            $errorInfo = $this->pdo->errorInfo();

            if ($errorInfo) {
                $result = 'SQLSTATE: ' . $errorInfo[0] . '<br/>';
                $result .= 'Code: ' . $errorInfo[1] . '<br/>';
                $result .= 'Message: ' . $errorInfo[2] . '<br/>';
            }
        }

        return $result;
    }

    /**
     * @throws DBException
     */
    public function beginTransaction()
    {
        $this ->pdo ->beginTransaction();
    }

    /**
     * @throws DBException
     */
    public function commitTransaction()
    {
        $this ->pdo ->commit();
    }

    /**
     * @throws DBException
     */
    public function rollbackTransaction()
    {
        $this ->pdo ->rollBack();
    }

    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return str_replace("'", "''", $str);
    }

    /**
     * @param string $text
     * @return string;
     */
    public function castWideMemo($text)
    {
        return $text;
    }

    /**
     * @param string $text
     * @return string;
     */
    public function extractYear($text)
    {
        return $this ->sqlDialect ->extractYearSQL($text);
    }


    /**
     * @param string $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        return $this ->sqlDialect ->dataToTimestamp($str);
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        return $this ->sqlDialect ->timestampToSQL($timestamp);
    }

    /**
     * Prevede timestamp na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        return $this ->sqlDialect ->dateToSQL($timestamp);
    }

    /**
     * prevede cas na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        return $this ->sqlDialect ->timeToSQL($timestamp);
    }


    public function timestampToParam($timestamp)
    {
        if (!isset($timestamp)) {
            return null;
        }

        return date('Y-m-d H:i:s', $timestamp);
    }

    public function dateToParam($timestamp)
    {
        if (!isset($timestamp)) {
            return null;
        }

        return date('Y-m-d', $timestamp);
    }


    public function timeToParam($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
            return "1899-12-30 $h:$m:$s";
        }

        return '1899-12-30 ' . date('H:i:s', $timestamp);
    }


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)\
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        if ( !isset($str) ) return null;

        $timestr = strrchr($str, ' '); // vraci vse za posledni mezerou

        $poma = explode (':', $timestr);

        return $poma[0] + $poma[1] / 60.0 + $poma[2] / 3600.0 ;
    }



    function getBlobParam($data)
    {
        return new PdoBlobParameter('?', $data, $this ->sqlDialect ->getPdoParameterLobOptions());
        //return new PdoBlobParameter('CONVERT(varbinary(max),?)', $data, $this ->sqlDialect ->getPdoParameterLobOptions());
        //return new BlobParameter('?', $data);
    }
}


class PdoCursor extends ArrayCursor
{
    /**
     * @var \PDOStatement
     */
    //protected $pdoStatement;

    /**
     * @param PdoDatabase $database
     * @param \PDOStatement $pdoStatement
     */
    function __construct($database, $pdoStatement)
    {
        parent::__construct(null);
        $this ->database = $database;
        //$this->pdoStatement = $pdoStatement;

        if ($pdoStatement) {
            $this->data = $pdoStatement->fetchAll(\PDO::FETCH_NUM);
            $pdoStatement ->closeCursor();
        }
    }



//    /**
//     * @return array|bool
//     */
//    public function fetch()
//    {
//        return $this ->pdoStatement ->fetch(\PDO::FETCH_NUM);
//    }
//
//    /**
//     * Vraci pocet nactenych zaznamu
//     * @return int
//     */
//    public function numRows()
//    {
//        return $this ->pdoStatement ->rowCount(); // pozor! u nekterych databazi nemusi fungovat!
//    }

}


class PdoBlobParameter extends BlobParameter
{
    public $driverOptions;

    public function __construct($placeholder, $data, $driverOptions = null)
    {
        parent::__construct($placeholder, $data);
        $this ->driverOptions = $driverOptions;
    }


}


class PdoParameters extends PlaceholderParameters
{
    public function addParameter($value)
    {
        if ($value instanceof BlobParameter) {
            $this ->data[] = $value;
            return $value ->placeholder;
        }

        return parent::addParameter($value); // TODO: Change the autogenerated stub
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2302_169.zip: ronportal/ron/db/pdodatabase.php <==
<?php
/**
 * stanislav.elbl, 23.09.2021 12:10
 */

namespace Ron\DB;

use PDO;

class PdoDatabase extends Database
{
    /**
     * @var PDO
     */
    private $pdo;

    private $connectionString;
    private $attributes;

    /** @var SqlDialect */
    private $sqlDialect;
    
    public $lobParameterDriverOptions;

    /**
     * @param string $connectionString
     * @param SqlDialect $sqlDialect
     */
    public function __construct($connectionString, $sqlDialect)
    {
        $this ->connectionString = $connectionString;
        $this ->sqlDialect = $sqlDialect;

        $this ->concatenateOperator = $sqlDialect ->getConcatenateOperator();
        $this ->functions = $sqlDialect ->getReplaceFunctions();
    }


    /**
     * @return mixed
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return mixed
     */
    public function getAttributes()
    {
        return $this->attributes;
    }

    /**
     * @param mixed $attributes
     */
    public function setAttributes($attributes)
    {
        $this->attributes = $attributes;
    }



    /**
     * @return string
     */
    public function getDbType()
    {
        return $this ->sqlDialect ->getDbType();
    }


    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    public function connect($user, $password)
    {
        try {
            $this->pdo = new PDO($this->connectionString, $user, $password);
        } catch (\PDOException $e) {
            throw new DBException($this, '', $e ->getMessage(), $e, $e ->errorInfo);
        }

        if ($this ->attributes) {
            foreach ($this ->attributes as $attribute => $value)
                $this ->pdo->setAttribute($attribute, $value);
        }

        //$count = $this ->pdo ->query("SET NAMES '{$this ->codePage}'");
//        $count = $this ->pdo ->query("SET NAMES utf8");
//        if (!isset($count)) {
//            throw new \Exception('chybaaa!');
//        };
    }

    /**
     * @return bool
     */
    public function close()
    {
        return true;
    }


    public function createParameters()
    {
        return new PdoParameters($this);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return Cursor
     * @throws DBException
     */
    public function executeSelect($sqlText, $params = null)
    {
        if ($params) {
            $data = $this ->pdo ->prepare($sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }

            if (!$data ->execute($params ->getSqlParameters())) {
                throw new DBException($this, $sqlText);
            }
        } else {
            $data = $this->pdo->query($sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }
        }

        return new PdoCursor($this, $data);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @throws DBException
     */
    public function executeSql($sqlText, $params = null)
    {
        if ($params) {
            $data = $this ->pdo ->prepare($sqlText);

            if (!$data) {
                throw new DBException($this, $sqlText);
            }

            foreach ($params ->getSqlParameters() as $index => &$parameter) {
//                echo "<br/>parametr $index: "; var_dump($parameter); echo "<br/>";
                if (!isset($parameter)) {
                    $data->bindValue($index + 1, null, PDO::PARAM_NULL);
                } else if ($parameter instanceof PdoBlobParameter) {
                    $data->bindParam($index + 1, $parameter->data, PDO::PARAM_LOB, 0, $parameter ->driverOptions); //
                } else if (is_int($parameter)) {
                    $data->bindValue($index + 1, $parameter, PDO::PARAM_INT);
                } else {
                    $data->bindValue($index + 1, $parameter);
                }
            }

            if (!$data ->execute()) {
                throw new DBException($this, $sqlText);
            }

//            if (!$data ->execute($params ->getSqlParameters())) {
//                throw new DBException($this, $sqlText);
//            }

        } else {
            $count = $this->pdo->exec($sqlText);

            // count obsahuje pocet zmenenych zaznamu nebo NULL pri chybe
            if ($count === NULL ||  $count === false) {
                throw new DBException($this, $sqlText);
            }
        }
    }


    function lastAutoincrementValue()
    {
        return $this ->pdo ->lastInsertId();
    }

    /**
     * @return string
     */
    public function lastErrorText()
    {
        $result = '';

        if ($this ->pdo) {
            $errorInfo = $this->pdo->errorInfo();

            if ($errorInfo) {
                $result = 'SQLSTATE: ' . $errorInfo[0] . '<br/>';
                $result .= 'Code: ' . $errorInfo[1] . '<br/>';
                $result .= 'Message: ' . $errorInfo[2] . '<br/>';
            }
        }

        return $result;
    }

    /**
     * @throws DBException
     */
    public function beginTransaction()
    {
        $this ->pdo ->beginTransaction();
    }

    /**
     * @throws DBException
     */
    public function commitTransaction()
    {
        $this ->pdo ->commit();
    }

    /**
     * @throws DBException
     */
    public function rollbackTransaction()
    {
        $this ->pdo ->rollBack();
    }

    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return str_replace("'", "''", $str ?? '');
    }

    /**
     * @param string $text
     * @return string;
     */
    public function castWideMemo($text)
    {
        return $text;
    }

    /**
     * @param string $text
     * @return string;
     */
    public function extractYear($text)
    {
        return $this ->sqlDialect ->extractYearSQL($text);
    }


    /**
     * @param string $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        return $this ->sqlDialect ->dataToTimestamp($str);
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        return $this ->sqlDialect ->timestampToSQL($timestamp);
    }

    /**
     * Prevede timestamp na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        return $this ->sqlDialect ->dateToSQL($timestamp);
    }

    /**
     * prevede cas na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        return $this ->sqlDialect ->timeToSQL($timestamp);
    }


    public function timestampToParam($timestamp)
    {
        if (!isset($timestamp)) {
            return null;
        }

        return date('Y-m-d H:i:s', $timestamp);
    }

    public function dateToParam($timestamp)
    {
        if (!isset($timestamp)) {
            return null;
        }

        return date('Y-m-d', $timestamp);
    }


    public function timeToParam($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
            return "1899-12-30 $h:$m:$s";
        }

        return '1899-12-30 ' . date('H:i:s', $timestamp);
    }


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)\
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        if ( !isset($str) ) return null;

        $timestr = strrchr($str, ' '); // vraci vse za posledni mezerou

        $poma = explode (':', $timestr);

        return $poma[0] + $poma[1] / 60.0 + $poma[2] / 3600.0 ;
    }



    function getBlobParam($data)
    {
        return new PdoBlobParameter('?', $data, $this ->sqlDialect ->getPdoParameterLobOptions());
        //return new PdoBlobParameter('CONVERT(varbinary(max),?)', $data, $this ->sqlDialect ->getPdoParameterLobOptions());
        //return new BlobParameter('?', $data);
    }


    public function nextSequenceValueSQL($sequenceName)
    {
        return $this ->sqlDialect ->nextSequenceValueSQL($sequenceName); // TODO: doplnit parametr standalone?
    }


    /**
     * @param string $sequenceName
     * @return bool
     * @throws DBException
     */
    protected function sequenceExistsInternal(string $sequenceName):bool
    {
        $sql = $this ->sqlDialect ->sequenceExistsSQL($sequenceName);
        if ($sql) {
            $cursor = $this ->getSqlResult($sql);
            $row = $cursor ->fetch();
            $cursor ->close();

            return (bool)$row;
        }

        return false;
    }


}


class PdoCursor extends ArrayCursor
{
    /**
     * @var \PDOStatement
     */
    //protected $pdoStatement;

    /**
     * @param PdoDatabase $database
     * @param \PDOStatement $pdoStatement
     */
    function __construct($database, $pdoStatement)
    {
        parent::__construct(null);
        $this ->database = $database;
        //$this->pdoStatement = $pdoStatement;

        if ($pdoStatement) {
            $this->data = $pdoStatement->fetchAll(\PDO::FETCH_NUM);
            $pdoStatement ->closeCursor();
        }
    }



//    /**
//     * @return array|bool
//     */
//    public function fetch()
//    {
//        return $this ->pdoStatement ->fetch(\PDO::FETCH_NUM);
//    }
//
//    /**
//     * Vraci pocet nactenych zaznamu
//     * @return int
//     */
//    public function numRows()
//    {
//        return $this ->pdoStatement ->rowCount(); // pozor! u nekterych databazi nemusi fungovat!
//    }

}


class PdoBlobParameter extends BlobParameter
{
    public $driverOptions;

    public function __construct($placeholder, $data, $driverOptions = null)
    {
        parent::__construct($placeholder, $data);
        $this ->driverOptions = $driverOptions;
    }


}


class PdoParameters extends PlaceholderParameters
{
    public function addParameter($value)
    {
        if ($value instanceof BlobParameter) {
            $this ->data[] = $value;
            return $value ->placeholder;
        }

        return parent::addParameter($value); // TODO: Change the autogenerated stub
    }
}
